@@ -236,8 +236,9 @@ void accept_mutex_on()
 	continue;
 
     if (ret < 0) {
-	log_unixerr("fcntl", "F_SETLKW", "Error getting accept lock. Exiting!",
-		    server_conf);
+	log_unixerr("fcntl", "F_SETLKW", "Error getting accept lock. Exiting!"
+		    "Perhaps you need to use the LockFile directive to place "
+		    "your lock file on a local disk!", server_conf);
 	exit(1);
     }
 }
@@ -246,8 +247,9 @@ void accept_mutex_off()
 {
     if (fcntl (lock_fd, F_SETLKW, &unlock_it) < 0)
     {
-	log_unixerr("fcntl", "F_SETLKW", "Error freeing accept lock. Exiting!",
-		    server_conf);
+	log_unixerr("fcntl", "F_SETLKW", "Error freeing accept lock. Exiting!"
+		    "Perhaps you need to use the LockFile directive to place "
+		    "your lock file on a local disk!", server_conf);
 	exit(1);
     }
 }
@@ -1712,7 +1714,9 @@ void child_main(int child_num_arg)
      */
     ap_setjmp (jmpbuffer);
 #ifndef __EMX__
+#ifdef SIGURG
     signal(SIGURG, timeout);
+#endif
 #endif    
 
     while (1) {
@@ -1776,8 +1780,16 @@ void child_main(int child_num_arg)
                 exit(0);
 
             errno = errsave;
-            if (srv < 0 && errno != EINTR)
+            if (srv < 0 && errno != EINTR) {
+		/* Single Unix documents select as returning errnos
+		 * EBADF, EINTR, and EINVAL... and in none of those
+		 * cases does it make sense to continue.  In fact
+		 * on Linux 2.0.x we seem to end up with EFAULT
+		 * occasionally, and we'd loop forever due to it.
+		 */
                 log_unixerr("select", "(listen)", NULL, server_conf);
+		exit(1);
+	    }
 
             if (srv <= 0)
                 continue;
@@ -1807,15 +1819,62 @@ void child_main(int child_num_arg)
             if (csd >= 0)
                 break;      /* We have a socket ready for reading */
             else {
-
-#if defined(EPROTO) && defined(ECONNABORTED)
-              if ((errno != EPROTO) && (errno != ECONNABORTED))
-#elif defined(EPROTO)
-              if (errno != EPROTO)
-#elif defined(ECONNABORTED)
-              if (errno != ECONNABORTED)
+		/* Our old behaviour here was to continue after accept()
+		 * errors.  But this leads us into lots of troubles
+		 * because most of the errors are quite fatal.  For
+		 * example, EMFILE can be caused by slow descriptor
+		 * leaks (say in a 3rd party module, or libc).  It's
+		 * foolish for us to continue after an EMFILE.  We also
+		 * seem to tickle kernel bugs on some platforms which
+		 * lead to never-ending loops here.  So it seems best
+		 * to just exit in most cases.
+		 */
+                switch (errno) {
+#ifdef EPROTO
+		    /* EPROTO on certain older kernels really means
+		     * ECONNABORTED, so we need to ignore it for them.
+		     * See discussion in new-httpd archives nh.9701
+		     * search for EPROTO.
+		     *
+		     * Also see nh.9603, search for EPROTO:
+		     * There is potentially a bug in Solaris 2.x x<6,
+		     * and other boxes that implement tcp sockets in
+		     * userland (i.e. on top of STREAMS).  On these
+		     * systems, EPROTO can actually result in a fatal
+		     * loop.  See PR#981 for example.  It's hard to
+		     * handle both uses of EPROTO.
+		     */
+                case EPROTO:
 #endif
-                log_unixerr("accept", "(client socket)", NULL, server_conf);
+#ifdef ECONNABORTED
+                case ECONNABORTED:
+#endif
+		    /* Linux generates the rest of these, other tcp
+		     * stacks (i.e. bsd) tend to hide them behind
+		     * getsockopt() interfaces.  They occur when
+		     * the net goes sour or the client disconnects
+		     * after the three-way handshake has been done
+		     * in the kernel but before userland has picked
+		     * up the socket.
+		     */
+#ifdef ECONNRESET
+                case ECONNRESET:
+#endif
+#ifdef ETIMEDOUT
+                case ETIMEDOUT:
+#endif
+#ifdef EHOSTUNREACH
+		case EHOSTUNREACH:
+#endif
+#ifdef ENETUNREACH
+		case ENETUNREACH:
+#endif
+                    break;
+
+		default:
+		    log_unixerr("accept", "(client socket)", NULL, server_conf);
+		    exit(1);
+		}
             }
 
 	    /* go around again, safe to die */
@@ -2463,6 +2522,9 @@ main(int argc, char *argv[])
           GETUSERMODE();
       }
 #endif
+        if (ap_setjmp (jmpbuffer)) {
+	    exit (0);
+        }
 
 	c = sizeof(sa_client);
 	if ((getpeername(fileno(stdin), &sa_client, &c)) < 0)
