@@ -129,7 +129,7 @@ int set_byterange (request_rec *r)
     if (!(range = table_get(r->headers_in, "Range")))
           range = table_get(r->headers_in, "Request-Range");
 
-    if (!range || strncmp(range, "bytes=", 6)) {
+    if (!range || strncasecmp(range, "bytes=", 6)) {
 	table_set (r->headers_out, "Accept-Ranges", "bytes");
 	return 0;
     }
@@ -339,7 +339,8 @@ int set_keepalive(request_rec *r)
      * as HTTP/1.0, but pass our request along with our HTTP/1.1 tag
      * to a HTTP/1.1 client. Better safe than sorry.
      */
-    table_merge(r->headers_out, "Connection", "close");
+    if (!wimpy)
+	table_merge(r->headers_out, "Connection", "close");
 
     r->connection->keepalive = 0;
 
@@ -547,13 +548,15 @@ void parse_uri (request_rec *r, const char *uri)
     }
 }
 
-const char *check_fulluri (request_rec *r, const char *uri) {
+const char *check_fulluri (request_rec *r, const char *uri)
+{
   char *name, *host;
   int i;
   unsigned port;
+  server_addr_rec * sar;
 
   /* This routine parses full URLs, if they match the server */
-  if (strncmp(uri, "http://", 7)) return uri;
+  if (strncasecmp(uri, "http://", 7)) return uri;
   name = pstrdup(r->pool, uri + 7);
   
   /* Find the hostname, assuming a valid request */
@@ -566,7 +569,13 @@ const char *check_fulluri (request_rec *r, const char *uri) {
   else port = 80;
 
   /* Make sure ports patch */
-  if (port != r->server->port) return uri;
+  if (port != r->server->port) {
+    for (sar = r->server->addrs; sar; sar = sar->next) {
+      if( (sar->host_port == 0) || (port == sar->host_port) )
+        break;
+    }
+    if (!sar) return uri;
+  }
 
   /* Save it for later use */
   r->hostname = pstrdup(r->pool, host);
@@ -677,15 +686,29 @@ void get_mime_headers (request_rec *r)
     }
 }
 
-static void check_hostalias (request_rec *r) {
+#define ADDR_MATCHES(addr1,addr2) \
+   (addr1.s_addr == addr2.s_addr) || (addr1.s_addr == htonl(INADDR_ANY)) \
+        || (addr1.s_addr == DEFAULT_VHOST_ADDR)
+
+static void check_hostalias (request_rec *r)
+{
   const char *hostname=r->hostname;
   char *host = getword(r->pool, &hostname, ':');	/* Get rid of port */
   unsigned port = (*hostname) ? atoi(hostname) : 80;
-  server_rec *s;
+  server_rec *s = r->server;
+  server_addr_rec * sar;
   int l;
 
-  if (port && (port != r->server->port))
-    return;
+/* make sure the client can't spoof the port;
+ * have to check all possiblities to see if the server
+ * should be listening. */
+  if (port != r->server->port) {
+    for (sar = s->addrs; sar; sar = sar->next) {
+      if ( (port == sar->host_port) || (sar->host_port == 0) )
+        break;
+    }
+    if (!sar) return;
+  }
 
   l = strlen(host)-1;
   if ((host[l]) == '.') {
@@ -703,37 +726,68 @@ static void check_hostalias (request_rec *r) {
 	    configuration */
 	continue;
     }
+/* ok, now there are several possibilities, and we're matching the
+ * hostname, the port, and r->connection->local_addr.  The last is
+ * required so as to only respond on an address to which this vhost
+ * should actually be listening.
+ *
+ * Either we can match s->server_name and s->port while matching
+ * against the ip address in a record in the s->addrs list *or* we
+ * can match s->server_name and a complete record in the s->addrs
+ * list *or* we can match the virtual host name and the address and
+ * the port of a record in the s->addrs list.
+ */
+    if (!strcasecmp(host,s->server_hostname)) {   /* ServerName matches hostname */
+      if (port == s->port) {                  /* possibly configured by Port */
+        for (sar = s->addrs; sar; sar = sar->next) {
+          if (ADDR_MATCHES(sar->host_addr,r->connection->local_addr.sin_addr))
+            break; /* SN matches, Port matches, and one IP addr matches */
+        }
+      } else {                             /* check to see if an addr matches */
+        for (sar = s->addrs; sar; sar = sar->next) {
+          if (((port == sar->host_port) || (sar->host_port == 0))
+              && (ADDR_MATCHES(sar->host_addr,
+                               r->connection->local_addr.sin_addr)))
+            break; /* SN matches, and a addr matches IP & port */
+        }
+      }
+      if (sar) { /* we got a match */
+        r->server = r->connection->server = s;
+        if (r->hostlen && !strncasecmp(r->uri, "http://", 7)) {
+          r->uri += r->hostlen;
+          parse_uri(r, r->uri);
+          /* we still might want to do something below (ie. set r->proxyreq) */
+        }
+      }
+    } /* ServerName doesn't match */
 
-    if ((!strcasecmp(host, s->server_hostname)) && (port == s->port)) {
-      r->server = r->connection->server = s;
-      if (r->hostlen && !strncmp(r->uri, "http://", 7)) {
-	r->uri += r->hostlen;
-	parse_uri(r, r->uri);
+    /* now s->addrs list, include the names, from the VirtualHost directive */
+    for (sar = s->addrs; sar; sar = sar->next) {
+      if (((sar->host_port==0) || (port==sar->host_port))
+          && (ADDR_MATCHES(sar->host_addr,r->connection->local_addr.sin_addr))
+          && !strcasecmp(host,sar->virthost)) {
+        /* ok, an element in the addrs list matched all three items */
+        r->server = r->connection->server = s;
+        if (r->hostlen && !strncasecmp(r->uri, "http://", 7)) {
+          r->uri += r->hostlen;
+          r->proxyreq = 0;
+        }
       }
     }
 
-    /* search all the names from <VirtualHost> directive */
-    for( sar = s->addrs; sar; sar = sar->next ) {
-      if( !strcasecmp( sar->virthost, host ) &&
-       ( (sar->host_port == 0) || (port == sar->host_port) )) {
-	r->server = r->connection->server = s;
-	if( r->hostlen && !strncmp( r->uri, "http://", 7) ) {
-	  r->uri += r->hostlen;
-	  r->proxyreq = 0;
-	}
-      }
-    }
-
-    /* search all the aliases from ServerAlias directive */
+    /* search all the aliases from ServerAlias directive
+     * ServerAlias acts like a wildcard, so as to help deal with the
+     * transition when the DNS for a given host changes.
+     */
     names = s->names;
-    if( names ) {
+    if (names) {
       while (*names) {
 	char *name = getword_conf (r->pool, &names);
 
 	if ((is_matchexp(name) && !strcasecmp_match(host, name)) ||
 	    (!strcasecmp(host, name))) {
 	  r->server = r->connection->server = s;
-	  if (r->hostlen && !strncmp(r->uri, "http://", 7)) {
+	  if (r->hostlen && !strncasecmp(r->uri, "http://", 7)) {
 	    r->uri += r->hostlen;
 	    r->proxyreq = 0;
 	  }
@@ -743,8 +797,11 @@ static void check_hostalias (request_rec *r) {
   }
 }
 
-void check_serverpath (request_rec *r) {
+void check_serverpath (request_rec *r)
+{
   server_rec *s;
+  server_addr_rec * sar = NULL;
+  int port = r->connection->local_addr.sin_port;
 
   /* This is in conjunction with the ServerPath code in
    * http_core, so we get the right host attached to a non-
@@ -752,10 +809,35 @@ void check_serverpath (request_rec *r) {
    */
 
   for (s = r->server->next; s; s = s->next) {
-    if (s->addrs && s->path && !strncmp(r->uri, s->path, s->pathlen) &&
-	(s->path[s->pathlen - 1] == '/' ||
-	 r->uri[s->pathlen] == '/' ||
-	 r->uri[s->pathlen] == '\0'))
+    /* we should check to make sure that this server should be listening
+     * at all.
+     *
+     * this code is duplicated in check_hostalias, but only one of these
+     * two functions runs for a given request.
+     */
+    if (!s->addrs) continue;
+
+    if( (port == s->port) ) {
+        for(sar = s->addrs; sar; sar = sar->next) {
+          if(ADDR_MATCHES(sar->host_addr, r->connection->local_addr.sin_addr))
+            break;
+        }
+    }
+    else {
+        for(sar = s->addrs; sar; sar = sar->next) {
+          if( ( (port == sar->host_port) || (sar->host_port == 0) )
+              && ( ADDR_MATCHES(sar->host_addr,
+                                r->connection->local_addr.sin_addr) ) )
+            break;
+        }
+    }
+
+    if (!sar) continue; /* no match */
+
+    if (s->path && !strncmp(r->uri, s->path, s->pathlen)
+	&& (s->path[s->pathlen - 1] == '/'
+	    || r->uri[s->pathlen] == '/'
+	    || r->uri[s->pathlen] == '\0'))
       r->server = r->connection->server = s;
   }
 }
@@ -927,7 +1009,7 @@ int get_basic_auth_pw (request_rec *r, char **pw)
 	return AUTH_REQUIRED;
     }
 
-    if (strcmp(getword (r->pool, &auth_line, ' '), "Basic")) {
+    if (strcasecmp(getword (r->pool, &auth_line, ' '), "Basic")) {
         /* Client tried to authenticate using wrong auth scheme */
         log_reason ("client used wrong authentication scheme", r->uri, r);
         note_basic_auth_failure (r);
@@ -935,7 +1017,7 @@ int get_basic_auth_pw (request_rec *r, char **pw)
     }
 
     t = uudecode (r->pool, auth_line);
-    r->connection->user = getword_nulls_nc (r->pool, &t, ':');
+    r->connection->user = getword_nulls_nc (r->connection->pool, &t, ':');
     r->connection->auth_type = "Basic";
 
     *pw = t;
@@ -1091,7 +1173,7 @@ static void terminate_header (BUFF *client)
     long int bs;
 
     bgetopt(client, BO_BYTECT, &bs);
-    if (bs == 256 || bs == 257)
+    if (bs >= 255 && bs <= 257)
         bputs("X-Pad: avoid browser bug\015\012", client);
 
     bputs("\015\012", client);    /* Send the terminating empty line */
@@ -1352,7 +1434,9 @@ int setup_client_block (request_rec *r, int read_policy)
 
 int should_client_block (request_rec *r)
 {
-    if (r->read_length || is_HTTP_ERROR(r->status))
+    /* First check if we have already read the request body */
+  
+    if (r->read_length || (!r->read_chunked && (r->remaining <= 0)))
         return 0;
 
     if (!r->read_chunked && (r->remaining <= 0))
@@ -1587,12 +1671,12 @@ long send_fd_length(FILE *f, request_rec *r, long length)
             break;
         }
         o=0;
-	total_bytes_sent += n;
 
         while (n && !r->connection->aborted) {
             w = bwrite(r->connection->client, &buf[o], n);
             if (w > 0) {
                 reset_timeout(r); /* reset timeout after successful write */
+		total_bytes_sent += w;
                 n-=w;
                 o+=w;
             }
