@@ -481,7 +481,7 @@ char *find_title(request_rec *r) {
     if (r->status != HTTP_OK) {
 	return NULL;
     }
-    if (r->content_type && !strcmp(r->content_type,"text/html") && !r->content_encoding) {
+    if (r->content_type && !strcasecmp(r->content_type,"text/html") && !r->content_encoding) {
         if(!(thefile = pfopen(r->pool, r->filename,"r")))
             return NULL;
         n = fread(titlebuf,sizeof(char),MAX_STRING_LEN - 1,thefile);
@@ -872,6 +872,7 @@ int handle_dir (request_rec *r)
     if (error_notfound)
 	return error_notfound;
 
+    r->allowed |= (1 << M_GET);
     if (r->method_number != M_GET) return NOT_IMPLEMENTED;
     
     /* OK, nothing easy.  Trot out the heavy artillery... */
