@@ -1,6 +1,5 @@
-
 /* ====================================================================
- * Copyright (c) 1995 The Apache Group.  All rights reserved.
+ * Copyright (c) 1995-1997 The Apache Group.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -52,7 +51,6 @@
  */
 
 
-
 #include "httpd.h"
 #include "http_config.h"
 
@@ -84,7 +82,7 @@ void *make_referer_log_state (pool *p, server_rec *s)
     return (void *)cls;
 }
 
-char *set_referer_log (cmd_parms *parms, void *dummy, char *arg)
+const char *set_referer_log (cmd_parms *parms, void *dummy, char *arg)
 {
     referer_log_state *cls = get_module_config (parms->server->module_config,
 					       &referer_log_module);
@@ -93,7 +91,7 @@ char *set_referer_log (cmd_parms *parms, void *dummy, char *arg)
     return NULL;
 }
 
-char *add_referer_ignore (cmd_parms *parms, void *dummy, char *arg)
+const char *add_referer_ignore (cmd_parms *parms, void *dummy, char *arg)
 {
   char **addme;
   referer_log_state *cls = get_module_config (parms->server->module_config,
@@ -121,7 +119,13 @@ void referer_log_child (void *cmd)
     
     cleanup_for_exec();
     signal (SIGHUP, SIG_IGN);
+#ifdef __EMX__
+    /* For OS/2 we need to use a '/' */
+    execl (SHELL_PATH, SHELL_PATH, "/c", (char *)cmd, NULL);
+#else
     execl (SHELL_PATH, SHELL_PATH, "-c", (char *)cmd, NULL);
+#endif
+    perror ("execl");
     fprintf (stderr, "Exec of shell for logging failed!!!\n");
     exit (1);
 }
@@ -138,10 +142,9 @@ void open_referer_log (server_rec *s, pool *p)
     if (*cls->fname == '|') {
 	FILE *dummy;
 	
-	spawn_child(p, referer_log_child, (void *)(cls->fname+1),
-		    kill_after_timeout, &dummy, NULL);
-
-	if (dummy == NULL) {
+	if (!spawn_child (p, referer_log_child, (void *)(cls->fname+1),
+		    kill_after_timeout, &dummy, NULL)) {
+	    perror ("spawn_child");
 	    fprintf (stderr, "Couldn't fork child for RefererLog process\n");
 	    exit (1);
 	}
@@ -150,8 +153,8 @@ void open_referer_log (server_rec *s, pool *p)
     }
     else if(*cls->fname != '\0') {
       if((cls->referer_fd = popenf(p, fname, xfer_flags, xfer_mode)) < 0) {
-        fprintf(stderr,"httpd: could not open referer log file %s.\n", fname);
         perror("open");
+        fprintf(stderr,"httpd: could not open referer log file %s.\n", fname);
         exit(1);
       }
     }
@@ -227,5 +230,6 @@ module referer_log_module = {
    NULL,			/* check access */
    NULL,			/* type_checker */
    NULL,			/* fixups */
-   referer_log_transaction	/* logger */
+   referer_log_transaction,	/* logger */
+   NULL				/* header parser */
 };
