@@ -216,12 +216,12 @@ proxy_handler(request_rec *r)
     {
 	p = strchr(ents[i].scheme, ':');  /* is it a partial URL? */
 	if (strcmp(ents[i].scheme, "*") == 0 || 
-	    (p == NULL && strcmp(scheme, ents[i].scheme) == 0) ||
+	    (p == NULL && strcasecmp(scheme, ents[i].scheme) == 0) ||
 	    (p != NULL &&
-	       strncmp(url, ents[i].scheme, strlen(ents[i].scheme)) == 0))
+	       strncasecmp(url, ents[i].scheme, strlen(ents[i].scheme)) == 0))
 	{
 /* we only know how to handle communication to a proxy via http */
-	    if (strcmp(ents[i].protocol, "http") == 0)
+	    if (strcasecmp(ents[i].protocol, "http") == 0)
 		rc = proxy_http_handler(r, cr, url, ents[i].hostname,
 		    ents[i].port);
 	    else rc = DECLINED;
@@ -239,9 +239,9 @@ proxy_handler(request_rec *r)
     /* handle the scheme */
     if (r->method_number == M_CONNECT)
 	return proxy_connect_handler(r, cr, url);
-    if (strcmp(scheme, "http") == 0)
+    if (strcasecmp(scheme, "http") == 0)
 	return proxy_http_handler(r, cr, url, NULL, 0);
-    if (strcmp(scheme, "ftp") == 0)
+    if (strcasecmp(scheme, "ftp") == 0)
 	return proxy_ftp_handler(r, cr, url);
     else return NOT_IMPLEMENTED;
 }
@@ -301,7 +301,7 @@ add_proxy(cmd_parms *cmd, void *dummy, char *f, char *r)
     {
 	int i;
 	for (i=0; defports[i].scheme != NULL; i++)
-	    if (strcmp(defports[i].scheme, r) == 0) break;
+	    if (strcasecmp(defports[i].scheme, r) == 0) break;
 	port = defports[i].port;
     }
 
