@@ -296,8 +296,11 @@ proxy_date_canon(pool *p, char *x)
     for (mon=0; mon < 12; mon++) if (strcmp(month, months[mon]) == 0) break;
     if (mon == 12) return x;
 
-    if (strlen(x) < 31) x = palloc(p, 31);
-    ap_snprintf(x, strlen(x)+1, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT", wday[wk], mday,
+    if (strlen(x)+1 < 30) {
+        x = palloc(p, 30);
+    }
+    /* format: "Wed, 17 Dec 1997 00:53:40 GMT" (29 chars data) */
+    ap_snprintf(x, 30, "%s, %.2d %s %d %.2d:%.2d:%.2d GMT", wday[wk], mday,
 	    months[mon], year, hour, min, sec);
     return x;
 }
