@@ -83,6 +83,7 @@ char *ht_time(pool *p, time_t t, const char *fmt, int gmt) {
 
     /* check return code? */
     strftime(ts,MAX_STRING_LEN,fmt,tms);
+    ts[MAX_STRING_LEN - 1] = '\0';
     return pstrdup (p, ts);
 }
 
@@ -328,14 +329,22 @@ void getparents(char *name)
     }
 } 
 
-void no2slash(char *name) {
-    register int x,y;
+void no2slash(char *name)
+{
+    char *d, *s;
 
-    for(x=0; name[x];)
-        if(x && (name[x-1] == '/') && (name[x] == '/'))
-            for(y=x+1;name[y-1];y++)
-                name[y-1] = name[y];
-	else x++;
+    s = d = name;
+    while (*s) {
+        if ((*d++ = *s) == '/') {
+            do {
+                ++s;
+            } while (*s == '/');
+        }
+        else {
+            ++s;
+        }
+    }
+    *d = '\0';
 }
 
 char *make_dirstr(pool *p, const char *s, int n) {
@@ -561,6 +570,11 @@ cfg_getline(char *s, int n, FILE *f) {
     if(c == EOF)
 	return 1;
 
+    if(n < 2) {
+	/* too small, assume caller is crazy */
+	return 1;
+    }
+
     while(1) {
         if((c == '\t') || (c == ' ')) {
             s[i++] = ' ';
@@ -570,7 +584,7 @@ cfg_getline(char *s, int n, FILE *f) {
         if(c == CR) {
             c = getc(f);
         }
-        if(c == EOF || c == 0x4 || c == LF || i == (n-1)) {
+        if(c == EOF || c == 0x4 || c == LF || i >= (n-2)) {
             /* blast trailing whitespace */
             while(i && (s[i-1] == ' ')) --i;
             s[i] = '\0';
@@ -1198,7 +1212,7 @@ static char *find_fqdn(pool *a, struct hostent *p) {
     if(ind(p->h_name,'.') == -1) {
         for(x=0;p->h_aliases[x];++x) {
             if((ind(p->h_aliases[x],'.') != -1) && 
-               (!strncmp(p->h_aliases[x],p->h_name,strlen(p->h_name))))
+               (!strncasecmp(p->h_aliases[x],p->h_name,strlen(p->h_name))))
                 return pstrdup(a, p->h_aliases[x]);
         }
         return NULL;
